//******************************************************************************
// Lab 6 '4619 USCI I2C Solution  - USCI_B0, I2C Master Receiver
//
// Instructions:
//  Complete the code below to configure the USCI for I2C master mode
//  - Setup the USCI for master I2C mode
//  - Select the clock source
//  - Release the USCI for operation
//
// Description: The 'FG4619 I2C master receives a 16-bit conversion result
// from an I2C slave. Every 2 secs based on Timer_A, a new result is
// queried by the master. When a preset threshold is exceeded result-to-result,
// the LED on P5.1 is enabled.
//
// ACLK = 32.768kHz, MCLK = SMCLK = default DCO ~1048k, SCL ~= 100kHz
//
//                             MASTER
//                          MSP430FG4619
//                       -----------------
//                   /|\|              XIN|-
//       SLAVE        | |                 |  32kHz xtal
//                    --|RST          XOUT|-
//                      |                 |
//       Slave Data|<-->|P3.1/UCB0SDA     |
//   Slave Clock In|<---|P3.2/UCB0SCL     |
//                      |             P5.1|-> LED
//
//******************************************************************************
#include "msp430xG46x.h"

unsigned int RxByteCtr;
unsigned int RxWord;

#define ADCDeltaOn 31                   // Conversion result threshold delta
static unsigned int LastADCVal;         // Holds conversion result
void main(void)
{ volatile unsigned int i;
  WDTCTL = WDTPW + WDTHOLD;             // Stop WDT

  // Wait for xtal to stabilize
  do
  {
    IFG1 &= ~OFIFG;                     // Clear OSCFault flag
    for (i = 0x47FF; i > 0; i--);       // Time for flag to set
  }
  while ((IFG1 & OFIFG));               // OSCFault flag still set?

  P1OUT = 0;                            // All P1.x reset
  P1IE = 0x03;                          // P1.0, P1.1 Interrupt enabled
  P1IES = 0x03;                         // P1.0, P1.1 hi/low edge
  P1DIR = 0xFC;                         // P1.0/1 = input (switches)
  P2OUT = 0;                            // All P2.x reset
  P2DIR = 0xDF;                         // UCA0RXD = inp
  P3OUT = 0x20;                         // All P3.x reset (3.5 = buzzer)
  P3DIR = 0xFF;                         // All P3.x outputs
  P4OUT = 0;                            // All P4.x reset
  P4DIR = 0xFF;                         // All P4.x outputs
  P5OUT = 0;                            // All P5.x reset
  P5DIR = 0xFF;                         // All P5.x outputs
  P6OUT = 0;                            // All P6.x reset
  P6DIR = 0xFF;                         // All P6.x outputs
  P7OUT = 0;                            // All P7.x reset
  P7DIR = 0xFF;                         // All P7.x outputs
  P8OUT = 0;                            // All P8.x reset
  P8DIR = 0xFF;                         // All P8.x outputs
  P9OUT = 0;                            // All P9.x reset
  P9DIR = 0xFF;                         // All P9.x outputs
  P10OUT = 0;                           // All P10.x reset
  P10DIR = 0xFF;                        // All P10.x outputs

  // LCD_A
  for( i = 0; i < 12; i ++)
  {
    LCDMEM[i+2] = 0;                    // Clear LCD memory
  }

  LCDACTL = LCDON + LCD4MUX + LCDFREQ_128; // 4mux LCD, ACLK/128
  LCDAPCTL0 = 0x7E;                     // Segments 4-27
  P5SEL = 0x1C;                         // COMx setup

  P3SEL |= 0x06;                        // Assign I2C pins to USCI_B0
  UCB0CTL1 |= _______;                  // Enable SW reset (why?)
  UCB0CTL0 = _________________________; // I2C Master, synchronous mode
  UCB0CTL1 = __________________;        // Use SMCLK, keep UCSWRST set
  UCB0BR0 = 11;                         // fSCL = SMCLK/11 = 95.3kHz
  UCB0BR1 = 0;
  UCB0I2CSA = 0x48;                     // Set slave address
  UCB0CTL1 &= ~_______;                 // Clear SW reset, resume operation
  UCB0I2CIE |= UCNACKIE;                // Interrupt on slave Nack
  IE2 |= UCB0RXIE;                      // Enable RX interrupt

  TACTL = TASSEL_1 + MC_2;              // ACLK, cont. mode

  while (1)
  {
    RxByteCtr = 2;                      // Load RX byte counter
    UCB0CTL1 |= UCTXSTT;                // Send I2C start condition
    _BIS_SR(LPM0_bits + GIE);           // Enter LPM0, enable interrupts
                                        // Remain in LPM0 until all data
                                        // is RX'd

    if (RxWord > LastADCVal + ADCDeltaOn) // Threshold exceeded?
      P5OUT |= 0x02;                    // P5.1 = 1
    else
      P5OUT &= ~0x02;                   // P5.1 = 0

    LastADCVal = RxWord;                // Keep latest result

    _BIC_SR(GIE);                       // Disable interrupts
    TACCTL0 |= CCIE;                    // TACCR0 interrupt enabled
    _BIS_SR(LPM3_bits + GIE);           // Enter LPM3, enable interrupts
                                        // Remain in LPM3 until TACCR0
                                        // interrupt occurs
    TACCTL0 &= ~CCIE;                   // TACCR0 interrupt disabled
  }
}

#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR(void)
{
  _BIC_SR_IRQ(LPM3_bits);               // Exit from LPM3 on RETI
}

// The USCIAB0TX_ISR is structured such that it can be used to receive any
// 2+ number of bytes by pre-loading RxByteCtr with the byte count.
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR(void)
{
  RxByteCtr--;                          // Decrement RX byte counter

  if (RxByteCtr)
  {
    RxWord = (unsigned int)UCB0RXBUF << 8; // Get received byte
    if (RxByteCtr == 1)                 // Only one byte left?
      UCB0CTL1 |= UCTXSTP;              // Generate I2C stop condition
  }
  else
  {
    RxWord |= UCB0RXBUF;                // Get final received byte,
                                        // Combine MSB and LSB

    _BIC_SR_IRQ(LPM0_bits);             // Exit from LPM0 on RETI
  }
}

// The USCIAB0RX_ISR is used to handle the State flags of the I2C
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR(void)
{
  if (UCNACKIFG & UCB0STAT)
  {
    UCB0CTL1 |= UCTXSTP;                // Generate I2C stop condition
    UCB0STAT &= ~UCNACKIFG;
    _BIC_SR_IRQ(LPM0_bits);             // Exit from LPM0 on RETI
    return;
  }
}
